package com.infosys.propertyeditor.ui.formattedtable;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.CaretEvent;
import javax.swing.filechooser.FileFilter;

public abstract class FormattedFileChooser
	extends JPanel
	implements FormattedFileInterface {
	
	public static File DEFAULT_FILE = null;

	private Object o;
	private JComponent jValue;
	private FormattedButton jbChoose;
	private int fileType;
	private String dialogTitle = null;

	public FormattedFileChooser(
		Object o,
		int fileType,
		int columnWidth,
		boolean isRenderer) {

		this.o = o;
		this.fileType = fileType;
		
		setLayout(new GridBagLayout());
		setBackground(Color.white);

		if (isRenderer) {
			jValue = new FormattedLabel(o, columnWidth);
			if (fileType == JFileChooser.FILES_ONLY)
				((FormattedLabel) jValue).setIcon(
					UIManager.getIcon("FileView.fileIcon"));
			else if (fileType == JFileChooser.DIRECTORIES_ONLY)
				((FormattedLabel) jValue).setIcon(
					UIManager.getIcon("FileView.directoryIcon"));

			inputVerifier((FormattedLabel) jValue);
		} else {
			jValue = new FormattedTextField(o) {
				public void caretUpdate(
					CaretEvent e,
					FormattedTextField textField) {
					FormattedFileChooser.this.caretUpdate(e, textField);
				}

			};
		}

		jbChoose = new FormattedButton("...", fileType);

		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.gridwidth = GridBagConstraints.RELATIVE;

		c.weightx = 1;
		c.weighty = 1;
		c.insets = new Insets(0, 2, 0, 0);
		add(jValue, c);
		c.weightx = 0;
		add(jbChoose, c);
	}

	public void setBackground(Color c) {
		super.setBackground(c);
		if (jValue != null)
			jValue.setBackground(c);
	}

	public void setForeground(Color c) {
		super.setForeground(c);
		if (jValue != null)
			jValue.setForeground(c);
	}

	public Object getObject() {
		return o;
	}

	public void setObject(Object otherObject) {
		o = otherObject;
	}

	public String getText() {
		if (jValue instanceof FormattedLabel)
			return ((FormattedLabel) jValue).getText();

		if (jValue instanceof FormattedTextField)
			return ((FormattedTextField) jValue).getText();

		return "";
	}

	public void setText(String s) {
		if (jValue instanceof FormattedLabel)
			 ((FormattedLabel) jValue).setText(s);

		if (jValue instanceof FormattedTextField)
			 ((FormattedTextField) jValue).setText(s);
	}

	public String toString() {
		return getText();
	}

	public FormattedButton getButton() {
		return jbChoose;
	}

	public void update() {
		if(o.toString() != getText()) setText(o.toString());
		jValue.requestFocus();
	}
	
	public void setDialogTitle(String s){
		dialogTitle = s;
	}
	
	public JFileChooser getFileChooserDialog() {
		return getFileChooserDialog(null);
	}

	public JFileChooser getFileChooserDialog(FileFilter filter) {
		
		JFileChooser fc = new JFileChooser();
		File tempFile = new File(o.toString());
		if(tempFile.exists()) fc.setCurrentDirectory(tempFile);
		else fc.setCurrentDirectory(DEFAULT_FILE);
		
		fc.setFileSelectionMode(fileType);
		fc.setFileFilter(filter);
		
		if (dialogTitle == null && fileType == JFileChooser.FILES_ONLY)
			fc.setDialogTitle("Choose the file");
		else if (
			dialogTitle == null && fileType == JFileChooser.DIRECTORIES_ONLY)
			fc.setDialogTitle("Choose the directory");
		else fc.setDialogTitle(dialogTitle);
		if (fc.showOpenDialog(SwingUtilities.windowForComponent(this))
			== JFileChooser.APPROVE_OPTION) {
			DEFAULT_FILE = fc.getSelectedFile();
			return fc;
		}

		return null;
	}

	public class FormattedButton extends JButton implements ActionListener {

		public FormattedButton(String s, int fileType) {
			super(s);
			setMargin(new Insets(2, 2, 2, 2));
			setFocusPainted(false);
			addActionListener(this);

		}

		public void actionPerformed(ActionEvent e) {
			fileChooserAction(e, FormattedFileChooser.this);
		}
	}
}

interface FormattedFileInterface {

	//Only useful when used as an editor
	public void caretUpdate(CaretEvent e, FormattedTextField textField);

	//	Only useful when used as an editor
	public void fileChooserAction(
		ActionEvent e,
		FormattedFileChooser formattedFileChooser);

	//	Only useful when used as a renderer
	public void inputVerifier(FormattedLabel label);

}